/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.util.LinkedList;
import java.util.List;

import org.sablecc.sablecc.analysis.DepthFirstAdapter;
import org.sablecc.sablecc.node.*;

public class RecursiveProductionsDetections extends DepthFirstAdapter
{
  public List<String> listOfRecursiveProds = new LinkedList<>();
  private String currentProd;

  @Override
  public void caseAProd(AProd node)
  {
    currentProd = node.getId().getText();
    if(!node.getId().getText().startsWith("$"))
    {
      PAlt[] alts = node.getAlts().toArray(new PAlt[0]);

      for(PAlt alt : alts)
      {
        alt.apply(this);
      }
    }
    else
    {
      listOfRecursiveProds.add( ResolveIds.name(currentProd) );
    }
  }

  @Override
  public void caseAAlt(AAlt node)
  {
    PElem[] temp = node.getElems().toArray(new PElem[0]);
    for(PElem elem : temp)
    {
      elem.apply(this);
    }
  }
  /*
  public void caseAIgnoredAlt(AIgnoredAlt node)
  {
  Object temp[] = node.getElems().toArray();
  for(int i = 0; i < temp.length; i++)
  {
    ((PElem) temp[i]).apply(this);
  }
  }
  */

  @Override
  public void caseAElem(AElem node)
  {
    if(node.getId().getText().equals(currentProd))
    {
      if(node.getSpecifier() != null && node.getSpecifier() instanceof ATokenSpecifier)
      {
        return;
      }
      if( !listOfRecursiveProds.contains(ResolveIds.name(currentProd)) )
      {
        listOfRecursiveProds.add( ResolveIds.name(currentProd) );
      }
    }
  }
}
